/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.stonehenge.stocktrader.services;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.stonehenge.stocktrader.CustomAccountBean;
import org.apache.stonehenge.stocktrader.CustomAccountProfileBean;
import org.apache.stonehenge.stocktrader.CustomHoldingBean;
import org.apache.stonehenge.stocktrader.CustomMarketSummaryBean;
import org.apache.stonehenge.stocktrader.CustomOrderBean;
import org.apache.stonehenge.stocktrader.CustomQuoteBean;
import org.apache.stonehenge.stocktrader.dal.DAOException;
import org.tempuri.TradeServiceWcfSkeleton;

import com.ibm.websphere.samples.trade.ArrayOfHoldingDataBean;
import com.ibm.websphere.samples.trade.ArrayOfOrderDataBean;
import com.ibm.websphere.samples.trade.Buy;
import com.ibm.websphere.samples.trade.BuyResponse;
import com.ibm.websphere.samples.trade.GetAccountData;
import com.ibm.websphere.samples.trade.GetAccountDataResponse;
import com.ibm.websphere.samples.trade.GetAccountProfileData;
import com.ibm.websphere.samples.trade.GetAccountProfileDataResponse;
import com.ibm.websphere.samples.trade.GetClosedOrders;
import com.ibm.websphere.samples.trade.GetClosedOrdersResponse;
import com.ibm.websphere.samples.trade.GetHolding;
import com.ibm.websphere.samples.trade.GetHoldingResponse;
import com.ibm.websphere.samples.trade.GetHoldings;
import com.ibm.websphere.samples.trade.GetHoldingsResponse;
import com.ibm.websphere.samples.trade.GetMarketSummary;
import com.ibm.websphere.samples.trade.GetMarketSummaryResponse;
import com.ibm.websphere.samples.trade.GetOrders;
import com.ibm.websphere.samples.trade.GetOrdersResponse;
import com.ibm.websphere.samples.trade.GetQuote;
import com.ibm.websphere.samples.trade.GetQuoteResponse;
import com.ibm.websphere.samples.trade.GetTopOrders;
import com.ibm.websphere.samples.trade.GetTopOrdersResponse;
import com.ibm.websphere.samples.trade.HoldingDataBean;
import com.ibm.websphere.samples.trade.IsOnline;
import com.ibm.websphere.samples.trade.Login;
import com.ibm.websphere.samples.trade.LoginResponse;
import com.ibm.websphere.samples.trade.Logout;
import com.ibm.websphere.samples.trade.LogoutResponse;
import com.ibm.websphere.samples.trade.OrderDataBean;
import com.ibm.websphere.samples.trade.Register;
import com.ibm.websphere.samples.trade.RegisterResponse;
import com.ibm.websphere.samples.trade.Sell;
import com.ibm.websphere.samples.trade.SellEnhanced;
import com.ibm.websphere.samples.trade.SellEnhancedResponse;
import com.ibm.websphere.samples.trade.SellResponse;
import com.ibm.websphere.samples.trade.UpdateAccountProfile;
import com.ibm.websphere.samples.trade.UpdateAccountProfileResponse;
import com.ibm.websphere.samples.trade.GetProfileIdFromStsIdentifier;
import com.ibm.websphere.samples.trade.GetProfileIdFromStsIdentifierResponse;
import java.util.Map;
import java.util.Iterator;
import java.util.HashMap;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axiom.soap.SOAPHeader;
import org.apache.axiom.soap.SOAPHeaderBlock;
import org.apache.axis2.context.MessageContext;
import javax.xml.namespace.QName;

public class TradeServiceImpl extends TradeServiceWcfSkeleton {
	private static final Log logger = LogFactory.getLog(TradeServiceImpl.class);

	private final TraderServiceManager mgr = new TraderServiceManager();

    private static final String WSSE_NS = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd";
    private static final String WSSE_LN = "Security";
    private static final String ASSERTION_TOKEN_IN = "Assertion";
    private static final String ATTRIBUTE_STATMENT_IN = "AttributeStatement";
    private static final String ATTRIBUTE_IN = "Attribute";

    private static final String ATTRIBUTE_NS = "AttributeNamespace";
    private static final String ATTRIBUTE_VALUE = "AttributeValue";
    private static final QName SEC_HEADER = new QName(WSSE_NS, WSSE_LN);
    public final static String SAML10_NS = "urn:oasis:names:tc:SAML:1.0:assertion";
    public final static String SAML11_NS = "http://docs.oasis-open.org/wss/oasis-wss-saml-token-profile-1.1#SAMLV1.1";
    public final static String SAML20_NS = "urn:oasis:names:tc:SAML:2.0:assertion";

    private String getProfileIdFromClaims() {
        Map<String, String> attributes = null;
        attributes = getSAMLAssertions(MessageContext.getCurrentMessageContext().getEnvelope());

        if (attributes != null) {
            for (String key : attributes.keySet()) {
                if (key.equals("http://schemas.xmlsoap.org/ws/2005/05/identity/claims")) {
                    return attributes.get(key);
                }
            }
        }

        return null;
    }

    private Map<String, String> getSAMLAssertions(SOAPEnvelope envelope) {
        SOAPHeaderBlock secHeader = null;
        OMElement assertionToken = null;

        secHeader = getSecHeader(envelope);
        if (secHeader == null) {
            return null;
        }

        String samlNameSpace = null;
        assertionToken = secHeader.getFirstChildWithName(new QName(SAML10_NS, ASSERTION_TOKEN_IN));
        if (assertionToken != null) {
            samlNameSpace = SAML10_NS;
        } else {
            assertionToken = secHeader.getFirstChildWithName(new QName(SAML11_NS,
                    ASSERTION_TOKEN_IN));
            if (assertionToken != null) {
                samlNameSpace = SAML11_NS;
            } else {
                assertionToken = secHeader.getFirstChildWithName(new QName(SAML20_NS,
                        ASSERTION_TOKEN_IN));
                if (assertionToken != null) {
                    samlNameSpace = SAML20_NS;
                }
            }
        }

        if (samlNameSpace == null) {
            logger.info("Unsupported SAML token type");
            return null;
        }

        if (assertionToken != null) {
            OMElement attributeStatementElem = assertionToken.getFirstChildWithName(new QName(
                    samlNameSpace, ATTRIBUTE_STATMENT_IN));
            if (attributeStatementElem != null) {
                Iterator<OMElement> attributes = null;
                attributes = attributeStatementElem.getChildrenWithName(new QName(samlNameSpace,
                        ATTRIBUTE_IN));
                if (attributes != null) {
                    Map<String, String> attributeMap;
                    attributeMap = new HashMap<String, String>();
                    while (attributes.hasNext()) {
                        OMElement attr = attributes.next();
                        OMElement attrValElement = null;
                        String attributeName = null;
                        String attributeValue = null;
                        attributeName = attr.getAttributeValue(new QName(ATTRIBUTE_NS));
                        attrValElement = attr.getFirstChildWithName(new QName(samlNameSpace,
                                ATTRIBUTE_VALUE));
                        attributeValue = attrValElement.getText();
                        attributeMap.put(attributeName, attributeValue);
                    }
                    return attributeMap;
                }
            }
        }
        return null;
    }

    private SOAPHeaderBlock getSecHeader(SOAPEnvelope envelope) {
        SOAPHeader header = envelope.getHeader();
        if (header != null) {
            return (SOAPHeaderBlock) header.getFirstChildWithName(SEC_HEADER);
        }
        return null;
    }

    public String getProfileIdFromStsIdentifier(String uniqueIdentifier, String issuingSts) {
        // GetProfileIdFromStsIdentifierResponse pid_r = new GetProfileIdFromStsIdentifierResponse();
        String pid = null;

        try {
            pid = mgr.getProfileIdFromStsIdentifier(uniqueIdentifier, issuingSts);
        } catch (DAOException e) {
            logger.debug("", e);
            throw new RuntimeException(e);
        }

        // pid_r.setGetProfileIdFromStsIdentifierReturn(pid);
        return pid;
    }

	public BuyResponse buy(Buy buy) {
		CustomOrderBean order;
		try {
            if (buy.getUserID() == null) {
                order = mgr.buy(getProfileIdFromClaims(), buy.getSymbol(), buy.getQuantity());
            } else {
			    order = mgr.buy(buy.getUserID(), buy.getSymbol(), buy.getQuantity());
            }
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		BuyResponse buyResponse = new BuyResponse();
		buyResponse.setBuyReturn(order);
		return buyResponse;
	}

	public GetAccountDataResponse getAccountData(GetAccountData getAccountData) {
		CustomAccountBean accountData;
		try {
            if (getAccountData.getUserID() == null) {
                accountData = mgr.getAccountData(getProfileIdFromClaims());
            } else {
			    accountData = mgr.getAccountData(getAccountData.getUserID());
            }
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		GetAccountDataResponse getAccountDataResponse = new GetAccountDataResponse();
		getAccountDataResponse.setGetAccountDataReturn(accountData);
		return getAccountDataResponse;
	}

	public GetAccountProfileDataResponse getAccountProfileData(GetAccountProfileData getAccountProfileData) {
		CustomAccountProfileBean accountProfileData;
		try {
            if (getAccountProfileData.getUserID() == null) {
                accountProfileData = mgr.getAccountProfileData(getProfileIdFromClaims());
            } else {
			    accountProfileData = mgr.getAccountProfileData(getAccountProfileData.getUserID());
            }
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		GetAccountProfileDataResponse response = new GetAccountProfileDataResponse();
		response.setGetAccountProfileDataReturn(accountProfileData);
		return response;
	}

	public GetClosedOrdersResponse getClosedOrders(GetClosedOrders getClosedOrders) {
		List<CustomOrderBean> closedOrders;
		try {
            if (getClosedOrders.getUserID() == null) {
                closedOrders = mgr.getClosedOrders(getProfileIdFromClaims());
            } else {
			    closedOrders = mgr.getClosedOrders(getClosedOrders.getUserID());
            }
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		GetClosedOrdersResponse response = new GetClosedOrdersResponse();
		ArrayOfOrderDataBean arrayOfOrderBean = new ArrayOfOrderDataBean();
		arrayOfOrderBean.setOrderDataBean(toArray(closedOrders));
		response.setGetClosedOrdersReturn(arrayOfOrderBean);
		return response;
	}

	public GetHoldingResponse getHolding(GetHolding getHolding) {
		CustomHoldingBean holding;
		try {
            if (getHolding.getUserID() == null) {
                holding = mgr.getHolding(getProfileIdFromClaims(), getHolding.getHoldingID());
            } else {
			    holding = mgr.getHolding(getHolding.getUserID(), getHolding.getHoldingID());
            }
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		GetHoldingResponse response = new GetHoldingResponse();
		response.setGetHoldingReturn(holding);
		return response;
	}

	public GetHoldingsResponse getHoldings(GetHoldings getHoldings) {
		List<CustomHoldingBean> holdingBeans;
		try {
            if (getHoldings.getUserID() == null) {
                holdingBeans = mgr.getHoldings(getProfileIdFromClaims());
            } else {
			    holdingBeans = mgr.getHoldings(getHoldings.getUserID());
            }
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		GetHoldingsResponse response = new GetHoldingsResponse();
		ArrayOfHoldingDataBean param = new ArrayOfHoldingDataBean();
		param.setHoldingDataBean(toHoldingsArray(holdingBeans));
		response.setGetHoldingsReturn(param);
		return response;
	}

	public GetMarketSummaryResponse getMarketSummary(GetMarketSummary getMarketSummary) {
		CustomMarketSummaryBean param;
		try {
			param = mgr.getMarketSummary();
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		GetMarketSummaryResponse response = new GetMarketSummaryResponse();
		response.setGetMarketSummaryReturn(param);
		return response;
	}

	public GetOrdersResponse getOrders(GetOrders getOrders) {
		List<CustomOrderBean> orders;
		try {
            if (getOrders.getUserID() == null) {
                orders = mgr.getOrders(getProfileIdFromClaims());
            } else {
			    orders = mgr.getOrders(getOrders.getUserID());
            }
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		GetOrdersResponse response = new GetOrdersResponse();
		ArrayOfOrderDataBean param = new ArrayOfOrderDataBean();
		param.setOrderDataBean(toArray(orders));
		response.setGetOrdersReturn(param);
		return response;
	}

	public GetQuoteResponse getQuote(GetQuote getQuote) {
		CustomQuoteBean param;
		try {
			param = mgr.getQuote(getQuote.getSymbol());
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		GetQuoteResponse response = new GetQuoteResponse();
		response.setGetQuoteReturn(param);
		return response;
	}

	public GetTopOrdersResponse getTopOrders(GetTopOrders getTopOrders) {
		List<CustomOrderBean> topOrders;
		try {
            if (getTopOrders.getUserID() == null) {
                topOrders = mgr.getTopOrders(getProfileIdFromClaims());
            } else {
			    topOrders = mgr.getTopOrders(getTopOrders.getUserID());
            }
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		GetTopOrdersResponse response = new GetTopOrdersResponse();
		ArrayOfOrderDataBean param = new ArrayOfOrderDataBean();
		param.setOrderDataBean(toArray(topOrders));
		response.setGetTopOrdersReturn(param);
		return response;
	}

	public void isOnline(IsOnline isOnline) {
		if (logger.isDebugEnabled()) {
			logger.debug("TradeServiceImpl.isOnline()");
		}
	}

	public LoginResponse login(Login login) {
		CustomAccountBean param;
		try {
            if (login.getUserID() == null) {
                param = mgr.login(getProfileIdFromClaims(), login.getPassword());
            } else {
			    param = mgr.login(login.getUserID(), login.getPassword());
            }
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		LoginResponse response = new LoginResponse();
		response.setLoginReturn(param);
		return response;
	}

	public LogoutResponse logout(Logout logout) {
		try {
            if (logout.getUserID() == null) {
                mgr.logout(getProfileIdFromClaims());
            } else {
			    mgr.logout(logout.getUserID());
            }
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		LogoutResponse response = new LogoutResponse();
		return response;
	}

	public RegisterResponse register(Register register) {
		CustomAccountBean param;
		try {
            String userId = (register.getUserID() == null) ? getProfileIdFromClaims()
                                                            : register.getUserID();
			param = mgr.register(
                    userId,
                    register.getPassword(),
					register.getFullname(),
                    register.getAddress(),
                    register.getEmail(),
                    register.getCreditcard(),
                    register.getOpenBalance());
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		RegisterResponse response = new RegisterResponse();
		response.setRegisterReturn(param);
		return response;
	}

	public SellResponse sell(Sell sell) {
		CustomOrderBean param;
		try {
            if (sell.getUserID() == null) {
                param = mgr.sell(getProfileIdFromClaims(), sell.getHoldingID(), sell.getOrderProcessingMode());
            } else {
			    param = mgr.sell(sell.getUserID(), sell.getHoldingID(), sell.getOrderProcessingMode());
            }
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		SellResponse response = new SellResponse();
		response.setSellReturn(param);
		return response;
	}

	public SellEnhancedResponse sellEnhanced(SellEnhanced sellEnhanced) {
		CustomOrderBean param;
		try {
            if (sellEnhanced.getUserID() == null) {
                param = mgr.sellEnhanced(getProfileIdFromClaims(), sellEnhanced.getHoldingID(), sellEnhanced.getQuantity());
            } else {
			    param = mgr.sellEnhanced(sellEnhanced.getUserID(), sellEnhanced.getHoldingID(), sellEnhanced.getQuantity());
            }
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		SellEnhancedResponse response = new SellEnhancedResponse();
		response.setSellEnhancedReturn(param);
		return response;
	}

	public UpdateAccountProfileResponse updateAccountProfile(UpdateAccountProfile updateAccountProfile) {
		CustomAccountProfileBean accountProfile = new CustomAccountProfileBean(updateAccountProfile.getProfileData());
		CustomAccountProfileBean param;
		try {
			param = mgr.updateAccountProfile(accountProfile);
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}

		UpdateAccountProfileResponse response = new UpdateAccountProfileResponse();
		response.setUpdateAccountProfileReturn(param);
		return response;
	}

	private OrderDataBean[] toArray(List<CustomOrderBean> orders) {
		OrderDataBean[] arrayOfOrders = new OrderDataBean[orders.size()];
		return (OrderDataBean[]) orders.toArray(arrayOfOrders);
	}

	private HoldingDataBean[] toHoldingsArray(List<CustomHoldingBean> holdings) {
		HoldingDataBean[] arrayOfHoldings = new HoldingDataBean[holdings.size()];
		return (HoldingDataBean[]) holdings.toArray(arrayOfHoldings);
	}
}
